"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.namedAnchors = exports.anchorPrefix = exports.preserveAnchorCasing = exports.publicPath = exports.textContentMappings = exports.indexFormat = exports.typeDeclarationFormat = exports.enumMembersFormat = exports.propertiesFormat = exports.parametersFormat = exports.expandObjects = exports.useCodeBlocks = exports.excludeGroups = exports.memberPageTitle = exports.indexPageTitle = exports.hideInPageTOC = exports.hideBreadcrumbs = exports.hidePageTitle = exports.hidePageHeader = exports.mergeReadme = exports.entryModule = exports.entryFileName = exports.outputFileStrategy = void 0;
const typedoc_1 = require("typedoc");
const maps_1 = require("./maps");
/**
 *
 * TypeDoc creates documentation according to exports derived from the given [`entryPointsStrategy`](https://typedoc.org/options/input/#entrypointstrategy) configuration.
 *
 * This option provides some flexibility as to how output files are generated with the following keywords:
 *
 * @members
 *
 * Generates an individual file for each exported member. This is the standard behaviour of the HTML theme and the plugin default.
 *
 * @modules
 *
 * Generates a single file for every Module or Namespace where all members are hoisted to a single module file. This creates a flat navigation structure and reduces the amount of files generated.
 *
 * @category Output
 */
exports.outputFileStrategy = {
    help: 'Determines how output files are generated.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.OutputFileStrategy,
    defaultValue: maps_1.OutputFileStrategy.Members,
};
/**
 * The entry page is the root page of the documentation, equivalent to `index.html` for web pages.
 *
 * `README.md` is recognised when browsing folders on repos and Wikis and is the plugin default. `index.md` might be more suitable for other use-cases.
 *
 * The content of this file will be resolved in the following order:
 *
 * 1. The value of the [`entryModule`](#entrymodule) option (if defined).
 * 2. The resolved Readme file (skipped if the TypeDoc [`readme`](https://typedoc.org/options/input/#readme) option is set to `none`).
 * 3. The documentation index page.
 *
 * @example "index.md"
 *
 * @category Output
 *
 */
exports.entryFileName = {
    help: 'The file name of the entry page.',
    type: typedoc_1.ParameterType.String,
    defaultValue: 'README.md',
};
/**
 * This option can be used when the root page of the documentation should be a specific module (typically a module named "index").
 *
 * This is only applicable when multiple modules are resolved.
 *
 * @example "index"
 *
 * @category Output
 */
exports.entryModule = {
    help: 'The name of a module that should act as the root page for the documentation.',
    type: typedoc_1.ParameterType.String,
};
/**
 * By default when a readme file is resolved, a seperate readme page is created. This option prepends the readme contents into the index page so only a single root page is genertaed.
 *
 * This option has no effect when [`readme`](https://typedoc.org/options/input/#readme) is set to `"none"`.
 *
 * @category Output
 *
 */
exports.mergeReadme = {
    help: 'Merges the resolved readme into the project index page.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * @category UI
 */
exports.hidePageHeader = {
    help: 'Do not print page header.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * @category UI
 */
exports.hidePageTitle = {
    help: 'Do not print page title.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * @category UI
 */
exports.hideBreadcrumbs = {
    help: 'Do not print breadcrumbs.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * @deprecated In-page TOC is now hidden as default.
 *
 * @category UI
 */
exports.hideInPageTOC = {
    help: 'Do not render in-page TOC items.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * Provides a mechanism to change the main project index page title.
 *
 * @deprecated Please use `textContentMappings`.
 *
 * @category UI
 *
 */
exports.indexPageTitle = {
    help: 'The title of project index page.',
    type: typedoc_1.ParameterType.String,
    defaultValue: '{ProjectName}',
};
/**
 * Provides a mechanism to configure the page title of members.
 *
 * @deprecated Please use `textContentMappings`.
 *
 * @category UI
 */
exports.memberPageTitle = {
    help: 'The page title of member pages.',
    type: typedoc_1.ParameterType.String,
    defaultValue: '{kind}: {name}',
};
/**
 * By default members are grouped by kind (eg Classes, Functions etc).
 *
 * This creates a flat structure where all members are displayed at the same heading level.
 *
 * @category UI
 */
exports.excludeGroups = {
    help: 'Excludes grouping by kind so all members are rendered and sorted at the same level.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * This option can be used to improve readability and aesthetics when defining signatures and declarations.
 *
 * Please note that when this option is set to `true` it is not possible to link to other references.
 *
 * As a work around the [`@link`](https://typedoc.org/tags/link/) tag can be be used to manually reference types.
 *
 * @category UI
 */
exports.useCodeBlocks = {
    help: 'Wraps signatures and declarations in code blocks.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * By default objects inside declarations are collapsed to preserve space and improve readability.
 *
 * This option should be set when a full object representation is preferred.
 *
 * @Default
 *
 * `Object`
 *
 * @Expanded
 *
 * \{ `x`: `string` }
 *
 * @category UI
 */
exports.expandObjects = {
    help: 'Expand objects inside declarations.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * This option either renders parameters for functions and class methods as a list or in tabular format.
 *
 * @category UI
 */
exports.parametersFormat = {
    help: 'Specify the render style of parameter and type parameter groups.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.FormatStyle,
    defaultValue: maps_1.FormatStyle.List,
};
/**
 *  This option either renders properties for classes and interfaces as a list or in tabular format.
 *
 * @category UI
 *
 */
exports.propertiesFormat = {
    help: 'Specify the render style of property groups for interfaces and classes.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.FormatStyle,
    defaultValue: maps_1.FormatStyle.List,
};
/**
 * This option either renders members of enums as a list or in tabular format.
 *
 * @category UI
 */
exports.enumMembersFormat = {
    help: 'Specify the render style of enumuration members.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.FormatStyle,
    defaultValue: maps_1.FormatStyle.List,
};
/**
 * This option either renders type declrations as a list or in tabular format.
 *
 * @category UI
 */
exports.typeDeclarationFormat = {
    help: 'Specify the render style for type declaration members.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.FormatStyle,
    defaultValue: maps_1.FormatStyle.List,
};
/**
 * This option renders index items either as a simple list or in a table with a description column exposing the comment summary.
 *
 * @category UI
 */
exports.indexFormat = {
    help: 'Specify the render format for index items.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.FormatStyle,
    defaultValue: maps_1.FormatStyle.List,
};
/**
 * This option enables changing static text rendered to the documentation.
 * Useful if an alternative English phrase is preferred or to translate English text to another language.
 * This option does not attempt to address translating text within code comments.
 *
 * **Placeholders**
 *
 * Default values within curly braces `{}` indicates a placeholder of dynamic text.
 * The `{version}` placeholder requires the TypeDoc option [`includeVersion`](https://typedoc.org/options/input/#includeversion) to be true.
 *
 * **keys**
 *
 * Keys are categorised with the following namespace conventions:
 *
 * - `header.*` defines text in the page header (if displayed).
 * - `breadcrumbs.*` defines breadcrumbs in page header (if displayed).
 * - `title.*` defines text in main page titles.
 * - `label.*` other text in page content, including headings and table headers.
 * - `kind.*` defines text mappings to TypeDoc's `ReflectionKind` definitions.
 *
 * Only keys that require translation need to be added to the object.
 *
 * @category UI
 */
exports.textContentMappings = {
    help: 'Provides a mechanism to change the content of text used in documentation.',
    type: typedoc_1.ParameterType.Mixed,
    defaultValue: maps_1.StaticText,
    validate(value) {
        if (!value || typeof value !== 'object') {
            throw new Error('textContentMappings must be an object.');
        }
        for (const val of Object.values(value)) {
            if (typeof val !== 'string') {
                throw new Error(`All values of textContentMappings must be strings.`);
            }
        }
    },
};
/**
 * If undefined all urls will be relative.
 *
 * @example "http://abc.com"
 *
 * @category Utility
 */
exports.publicPath = {
    help: 'Specify the base path for all urls.',
    type: typedoc_1.ParameterType.String,
    defaultValue: undefined,
};
/**
 * By default references to symbol anchor links are lowercased.
 *
 * This option can be used for engines that require the preservation of anchor link casing.
 *
 * @category Utility
 */
exports.preserveAnchorCasing = {
    help: 'Preserve anchor casing when generating link to symbols.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * This option should be used when parsers requiren a custom anchor prefix.
 *
 * @example "markdown-header"
 *
 * @category Utility
 */
exports.anchorPrefix = {
    help: 'Custom anchor prefix',
    type: typedoc_1.ParameterType.String,
    defaultValue: undefined,
};
/**
 * This option should be used if there are issues with anchoring to symbols within a page.
 *
 * - For markdown parsers that do not automatically assign header ids.
 * - When cross referencing symbols that are referenced in a table row.
 *
 * @category Utility
 */
exports.namedAnchors = {
    help: 'Add HTML named anchors to headings and table rows.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
