"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NavigationContext = void 0;
const path = __importStar(require("path"));
const typedoc_1 = require("typedoc");
const maps_1 = require("../options/maps");
class NavigationContext {
    constructor(theme, options, renderer) {
        this.theme = theme;
        this.options = options;
        this.renderer = renderer;
        this.navigation = [];
    }
    getNavigation(project) {
        var _a, _b;
        const isPackages = this.options.entryPointStrategy === typedoc_1.EntryPointStrategy.Packages;
        if (isPackages) {
            if (((_a = Object.keys(this.renderer.packages)) === null || _a === void 0 ? void 0 : _a.length) === 1) {
                this.buildNavigationFromProject(project);
            }
            else {
                (_b = project.children) === null || _b === void 0 ? void 0 : _b.forEach((projectChild) => {
                    this.buildNavigationFromPackage(projectChild);
                });
            }
        }
        else {
            this.buildNavigationFromProject(project);
        }
        return this.navigation;
    }
    buildNavigationFromPackage(projectChild) {
        const entryFileName = this.options.entryFileName;
        const preservePackageReadme = Boolean(projectChild.readme) && !this.options.mergeReadme;
        const packageMeta = this.renderer.packages[projectChild.name];
        const outputFileStrategy = (packageMeta === null || packageMeta === void 0 ? void 0 : packageMeta.outputFileStrategy) || this.options.outputFileStrategy;
        const projectChildUrl = preservePackageReadme
            ? `${path.dirname(projectChild.url)}/${entryFileName}`
            : projectChild.url;
        const isModulesGroup = (projectChild === null || projectChild === void 0 ? void 0 : projectChild.groups) &&
            (projectChild === null || projectChild === void 0 ? void 0 : projectChild.groups[0].children.every((child) => child.kindOf(typedoc_1.ReflectionKind.Module)));
        const children = [];
        if (preservePackageReadme &&
            !isModulesGroup &&
            outputFileStrategy === maps_1.OutputFileStrategy.Modules) {
            children.push({
                title: this.theme.getTextContent('label.globals'),
                url: projectChild.url,
            });
        }
        const childGroups = this.getChildrenOrGroups(projectChild, outputFileStrategy);
        if (childGroups) {
            children.push(...childGroups);
        }
        this.navigation.push({
            title: projectChild.name,
            children,
            ...(projectChildUrl && { url: projectChildUrl }),
        });
    }
    buildNavigationFromProject(project) {
        var _a, _b, _c, _d;
        if ((_a = project.groups) === null || _a === void 0 ? void 0 : _a.length) {
            const entryModule = Boolean((_b = project === null || project === void 0 ? void 0 : project.groups[0]) === null || _b === void 0 ? void 0 : _b.children.find((child) => child.name === this.options.entryModule));
            const isOnlyModules = (_c = project.children) === null || _c === void 0 ? void 0 : _c.every((child) => child.kindOf(typedoc_1.ReflectionKind.Module));
            if ((project.groups.length === 1 && !Boolean(entryModule)) ||
                isOnlyModules) {
                const children = this.getGroupChildren(project.groups[0]);
                if (children) {
                    this.navigation.push(...children.filter((child) => child.title !== this.options.entryModule));
                }
            }
            else {
                (_d = project.groups) === null || _d === void 0 ? void 0 : _d.forEach((projectGroup) => {
                    const children = this.getGroupChildren(projectGroup);
                    const indexModule = projectGroup.children.find((child) => child.name === this.options.entryModule);
                    if (children.length) {
                        this.navigation.push({
                            title: projectGroup.title,
                            children: children.filter((child) => child.title !== this.options.entryModule),
                        });
                    }
                    if (indexModule) {
                        const children = this.getChildrenOrGroups(indexModule);
                        if (children) {
                            this.navigation.push(...children);
                        }
                    }
                });
            }
        }
    }
    getCategoryGroupChildren(group) {
        var _a;
        return (_a = group.children) === null || _a === void 0 ? void 0 : _a.filter((child) => child.hasOwnDocument).map((child) => {
            const children = this.getChildrenOrGroups(child);
            return {
                title: child.name,
                url: child.url,
                ...(children && { children }),
            };
        });
    }
    getGroupChildren(group, outputFileStrategy) {
        var _a, _b, _c;
        if ((_a = group === null || group === void 0 ? void 0 : group.categories) === null || _a === void 0 ? void 0 : _a.length) {
            return (_b = group.categories) === null || _b === void 0 ? void 0 : _b.map((category) => {
                return {
                    title: category.title,
                    children: this.getCategoryGroupChildren(category),
                };
            });
        }
        return (_c = group.children) === null || _c === void 0 ? void 0 : _c.filter((child) => child.hasOwnDocument).map((child) => {
            var _a, _b;
            const mapping = this.theme.getTemplateMapping(child.kind, outputFileStrategy);
            if (Boolean(mapping)) {
                const children = ((_a = child.categories) === null || _a === void 0 ? void 0 : _a.length)
                    ? (_b = child.categories) === null || _b === void 0 ? void 0 : _b.map((category) => {
                        const catChildren = this.getCategoryGroupChildren(category);
                        return catChildren.length
                            ? {
                                title: category.title,
                                children: catChildren,
                            }
                            : null;
                    }).filter((cat) => Boolean(cat))
                    : this.getChildrenOrGroups(child, outputFileStrategy);
                return {
                    title: child.name,
                    url: child.url,
                    ...(children && { children }),
                };
            }
        });
    }
    getChildrenOrGroups(reflection, outputFileStrategy) {
        var _a, _b, _c;
        if ((_a = reflection.groups) === null || _a === void 0 ? void 0 : _a.some((group) => group.allChildrenHaveOwnDocument())) {
            if (this.options.excludeGroups) {
                return (_b = reflection.children) === null || _b === void 0 ? void 0 : _b.filter((child) => child.hasOwnDocument).map((child) => {
                    const children = this.getChildrenOrGroups(child, outputFileStrategy);
                    return {
                        title: child.name,
                        url: child.url,
                        ...(children && { children }),
                    };
                });
            }
            const isModulesGroup = reflection.groups[0].children.every((child) => child.kindOf(typedoc_1.ReflectionKind.Module));
            if (isModulesGroup) {
                return (this.getGroupChildren(reflection.groups[0], outputFileStrategy) ||
                    null);
            }
            return (_c = reflection.groups) === null || _c === void 0 ? void 0 : _c.map((group) => {
                const groupChildren = this.getGroupChildren(group, outputFileStrategy);
                return groupChildren.length
                    ? {
                        title: group.title,
                        children: groupChildren || null,
                    }
                    : null;
            }).filter((group) => Boolean(group));
        }
        return null;
    }
}
exports.NavigationContext = NavigationContext;
