import { Options, Reflection, ReflectionKind } from 'typedoc';
import { TextContentMappings } from '../options/models';
import { MarkdownPageEvent } from '../plugin/events';
import { MarkdownTheme } from './theme';
/**
 * The render context of the {@link MarkdownTheme}.
 * This follows the implementation of TypeDocs [DefaultThemeRenderContext](https://typedoc.org/api/classes/DefaultThemeRenderContext.html)
 */
export declare class MarkdownThemeRenderContext {
    private theme;
    page: MarkdownPageEvent<Reflection> | null;
    options: Options;
    constructor(theme: MarkdownTheme, page: MarkdownPageEvent<Reflection> | null, options: Options);
    urlTo: (reflection: Reflection) => string | null;
    relativeURL: (url: string | undefined) => string | null;
    getTextContent(key: keyof TextContentMappings): string;
    groupTitle(title: string): string;
    kindString(kind: ReflectionKind): string;
    indexTitle(textContent: string, name: string, version?: string): string;
    parseUrl(url: string): string;
    /** @hidden */
    memberTemplate: (page: MarkdownPageEvent<import("typedoc").DeclarationReflection>) => string;
    /** @hidden */
    projectTemplate: (page: MarkdownPageEvent<import("typedoc").ProjectReflection>) => string;
    /** @hidden */
    readmeTemplate: (page: MarkdownPageEvent<import("typedoc").ProjectReflection>) => string;
    /** @hidden */
    reflectionTemplate: (page: MarkdownPageEvent<import("typedoc").DeclarationReflection>) => string;
    /** @hidden */
    breadcrumbs: (page: MarkdownPageEvent<import("typedoc").DeclarationReflection | import("typedoc").ProjectReflection>) => string;
    /** @hidden */
    commentParts: (parts: import("typedoc").CommentDisplayPart[]) => string;
    /** @hidden */
    comment: (comment: import("typedoc").Comment, headingLevel?: number | undefined, showSummary?: boolean | undefined, showTags?: boolean | undefined) => string;
    /** @hidden */
    footer: () => string;
    /** @hidden */
    header: (page: MarkdownPageEvent<import("typedoc").DeclarationReflection | import("typedoc").ProjectReflection>) => string;
    /** @hidden */
    pageIndex: (page: MarkdownPageEvent<import("typedoc").DeclarationReflection | import("typedoc").ProjectReflection>, headingLevel: number) => string;
    /** @hidden */
    reflectionIndex: (reflection: import("typedoc").DeclarationReflection | import("typedoc").ProjectReflection, inline: boolean | undefined, headingLevel: number) => string;
    /** @hidden */
    parametersList: (parameters: import("typedoc").ParameterReflection[]) => string;
    /** @hidden */
    typeParametersList: (typeParameters: import("typedoc").TypeParameterReflection[], headingLevel: number) => string;
    /** @hidden */
    accessorMember: (declaration: import("typedoc").DeclarationReflection, headingLevel: number) => string;
    /** @hidden */
    constructorMember: (reflection: import("typedoc").DeclarationReflection, headingLevel: number) => string;
    /** @hidden */
    declarationMemberIdentifier: (reflection: import("typedoc").DeclarationReflection) => string;
    /** @hidden */
    declarationMember: (declaration: import("typedoc").DeclarationReflection, headingLevel: number, nested?: boolean | undefined) => string;
    /** @hidden */
    memberHierarchy: (declarationHierarchy: import("typedoc").DeclarationHierarchy, headingLevel: number) => string;
    /** @hidden */
    indexSignatureTitle: (signature: import("typedoc").SignatureReflection) => string;
    /** @hidden */
    inheritance: (reflection: import("typedoc").DeclarationReflection | import("typedoc").SignatureReflection, headingLevel: number) => string;
    /** @hidden */
    referenceMember: (props: import("typedoc").ReferenceReflection) => string;
    /** @hidden */
    reflectionMember: (reflection: import("typedoc").DeclarationReflection, headingLevel: number) => string;
    /** @hidden */
    signatureMemberIdentifier: (signature: import("typedoc").SignatureReflection, opts?: {
        accessor?: string | undefined;
        includeType?: boolean | undefined;
    } | undefined) => string;
    /** @hidden */
    signatureMemberReturns: (signature: import("typedoc").SignatureReflection, headingLevel: number) => string;
    /** @hidden */
    signatureMember: (signature: import("typedoc").SignatureReflection, headingLevel: number, nested?: boolean | undefined, accessor?: string | undefined) => string;
    /** @hidden */
    sources: (reflection: import("typedoc").DeclarationReflection | import("typedoc").SignatureReflection, headingLevel: number) => string;
    /** @hidden */
    member: (reflection: import("typedoc").DeclarationReflection, headingLevel: number, nested?: boolean | undefined) => string;
    /** @hidden */
    typeDeclarationMember: (typeDeclaration: import("typedoc").DeclarationReflection, headingLevel: number) => string;
    /** @hidden */
    members: (container: import("typedoc").ContainerReflection, headingLevel: number) => string;
    /** @hidden */
    pageTitle: (page: MarkdownPageEvent<import("typedoc").DeclarationReflection | import("typedoc").ProjectReflection>) => string;
    /** @hidden */
    enumMembersTable: (props: import("typedoc").DeclarationReflection[]) => string;
    /** @hidden */
    parametersTable: (parameters: import("typedoc").ParameterReflection[]) => string;
    /** @hidden */
    propertiesTable: (props: import("typedoc").DeclarationReflection[], isEventProps?: boolean | undefined) => string;
    /** @hidden */
    typeDeclarationTable: (props: import("typedoc").DeclarationReflection[]) => string;
    /** @hidden */
    typeParametersTable: (typeParameters: import("typedoc").TypeParameterReflection[]) => string;
    /** @hidden */
    typeArguments: (typeArguments: import("typedoc").SomeType[], foreCollpase?: boolean | undefined) => string;
    /** @hidden */
    arrayType: (arrayType: import("typedoc").ArrayType) => string;
    /** @hidden */
    conditionalType: (conditionalType: import("typedoc").ConditionalType) => string;
    /** @hidden */
    declarationType: (declarationReflection: import("typedoc").DeclarationReflection) => string;
    /** @hidden */
    functionType: (modelSignatures: import("typedoc").SignatureReflection[]) => string;
    /** @hidden */
    indexAccessType: (model: import("typedoc").IndexedAccessType) => string;
    /** @hidden */
    inferredType: (model: import("typedoc").InferredType) => string;
    /** @hidden */
    intersectionType: (model: import("typedoc").IntersectionType) => string;
    /** @hidden */
    intrinsicType: (model: import("typedoc").IntrinsicType) => string;
    /** @hidden */
    literalType: (literalType: import("typedoc").LiteralType) => string;
    /** @hidden */
    namedTupleType: (member: import("typedoc").NamedTupleMember) => string;
    /** @hidden */
    queryType: (queryType: import("typedoc").QueryType) => string;
    /** @hidden */
    referenceType: (referenceType: import("typedoc").ReferenceType, foreCollpase?: boolean | undefined) => string;
    /** @hidden */
    reflectionType: (reflectionType: import("typedoc").ReflectionType, foreCollpase?: boolean | undefined) => string;
    /** @hidden */
    someType: (someType: import("typedoc").SomeType, foreCollpase?: boolean | undefined) => string;
    /** @hidden */
    tupleType: (tupleType: import("typedoc").TupleType) => string;
    /** @hidden */
    typeOperatorType: (model: import("typedoc").TypeOperatorType) => string;
    /** @hidden */
    unionType: (unionType: import("typedoc").UnionType) => string;
    /** @hidden */
    unknownType: (model: import("typedoc").UnknownType) => string;
}
