"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.commentParts = void 0;
/**
 * @category Partials
 */
function commentParts(context, parts) {
    const md = [];
    for (const part of parts) {
        switch (part.kind) {
            case 'text':
            case 'code':
                md.push(part.text);
                break;
            case 'inline-tag':
                switch (part.tag) {
                    case '@label':
                    case '@inheritdoc':
                        break;
                    case '@link':
                    case '@linkcode':
                    case '@linkplain': {
                        if (part.target) {
                            const url = getUrl(context, part);
                            const wrap = part.tag === '@linkcode' ? '`' : '';
                            md.push(url ? `[${wrap}${part.text}${wrap}](${url})` : part.text);
                        }
                        else {
                            md.push(part.text);
                        }
                        break;
                    }
                    default:
                        md.push(`{${part.tag} ${part.text}}`);
                        break;
                }
                break;
            default:
                md.push('');
        }
    }
    return md.join('');
}
exports.commentParts = commentParts;
function getUrl(context, part) {
    var _a, _b, _c, _d;
    if (part.target.url) {
        return context.relativeURL(part.target.url);
    }
    if ((_b = (_a = part.target) === null || _a === void 0 ? void 0 : _a.parent) === null || _b === void 0 ? void 0 : _b.url) {
        return context.relativeURL((_d = (_c = part.target) === null || _c === void 0 ? void 0 : _c.parent) === null || _d === void 0 ? void 0 : _d.url);
    }
    return part.target;
}
