"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.header = void 0;
const typedoc_1 = require("typedoc");
const elements_1 = require("../../../support/elements");
/**
 * @category Partials
 */
function header(context, page) {
    const isPackages = context.options.getValue('entryPointStrategy') ===
        typedoc_1.EntryPointStrategy.Packages;
    if (isPackages) {
        const packageItem = findPackage(page.model);
        if (packageItem) {
            return packageHeader(context, page);
        }
    }
    return projectHeader(context, page);
}
exports.header = header;
function projectHeader(context, page) {
    var _a, _b, _c;
    const entryFileName = context.options.getValue('entryFileName');
    const titleLink = context.options.getValue('titleLink');
    const md = [];
    const title = context.indexTitle(context.getTextContent('header.title'), page.project.name, page.project.packageVersion);
    const readmeLabel = context.getTextContent('header.readme');
    const indexLabel = context.getTextContent('header.docs');
    md.push(titleLink ? (0, elements_1.bold)((0, elements_1.link)(title, titleLink)) : (0, elements_1.bold)(title));
    md.push('•');
    const preserveReadme = Boolean(page.project.readme) && !context.options.getValue('mergeReadme');
    const preserveModulesPage = (((_a = page.project) === null || _a === void 0 ? void 0 : _a.groups) &&
        Boolean((_c = (_b = page.project) === null || _b === void 0 ? void 0 : _b.groups[0]) === null || _c === void 0 ? void 0 : _c.children.find((child) => child.name === context.options.getValue('entryModule')))) ||
        false;
    if (preserveReadme) {
        const links = [];
        if (page.url === entryFileName) {
            links.push(readmeLabel);
        }
        else {
            links.push((0, elements_1.link)(readmeLabel, context.relativeURL(preserveModulesPage ? 'readme_.md' : entryFileName)));
        }
        links.push('\\|');
        if (page.url === page.project.url) {
            links.push(indexLabel);
        }
        else {
            links.push((0, elements_1.link)(indexLabel, context.relativeURL(page.project.url)));
        }
        md.push(`${links.join(' ')}`);
    }
    else {
        md.push(`${indexLabel}`);
    }
    return `${md.join(' ')}\n\n***\n`;
}
function packageHeader(context, page) {
    const packageItem = findPackage(page.model);
    if (!packageItem) {
        return '';
    }
    const md = [];
    const readmeLabel = context.getTextContent('header.readme');
    const indexLabel = context.getTextContent('header.docs');
    const entryFileName = context.options.getValue('entryFileName');
    const packageItemName = packageItem.packageVersion
        ? `${packageItem.name} v${packageItem.packageVersion}`
        : packageItem.name;
    md.push((0, elements_1.bold)(packageItemName));
    md.push('•');
    const preservePackageReadme = Boolean(packageItem.readme) && !context.options.getValue('mergeReadme');
    if (preservePackageReadme) {
        const links = [];
        const readmeUrl = `${packageItem.name}/${entryFileName}`;
        if (page.url === readmeUrl) {
            links.push(readmeLabel);
        }
        else {
            links.push((0, elements_1.link)(readmeLabel, context.relativeURL(readmeUrl)));
        }
        links.push('\\|');
        if (page.url === packageItem.url) {
            links.push(indexLabel);
        }
        else {
            links.push((0, elements_1.link)(indexLabel, context.relativeURL(packageItem.url)));
        }
        md.push(`${links.join(' ')}`);
    }
    else {
        md.push(indexLabel);
    }
    return `${md.join(' ')}\n\n***\n`;
}
function findPackage(model) {
    var _a;
    if (model.kindOf(typedoc_1.ReflectionKind.Module) &&
        ((_a = model.parent) === null || _a === void 0 ? void 0 : _a.kindOf(typedoc_1.ReflectionKind.Project))) {
        return model;
    }
    if (model.parent) {
        return findPackage(model.parent);
    }
    return null;
}
