"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.reflectionIndex = void 0;
const typedoc_1 = require("typedoc");
const elements_1 = require("../../../support/elements");
const utils_1 = require("../../../support/utils");
function reflectionIndex(context, reflection, inline = false, headingLevel) {
    var _a;
    const md = [];
    const subHeadingLevel = headingLevel;
    if (reflection.categories) {
        reflection.categories.forEach((categoryGroup) => {
            md.push((0, elements_1.heading)(subHeadingLevel, categoryGroup.title) + '\n');
            md.push(getGroup(context, categoryGroup, inline) + '\n');
        });
    }
    else {
        const groups = (_a = reflection.groups) === null || _a === void 0 ? void 0 : _a.filter((group) => inline
            ? !group.allChildrenHaveOwnDocument()
            : group.allChildrenHaveOwnDocument());
        groups === null || groups === void 0 ? void 0 : groups.forEach((reflectionGroup) => {
            var _a;
            if (reflectionGroup.categories) {
                md.push((0, elements_1.heading)(subHeadingLevel, reflectionGroup.title) + '\n');
                reflectionGroup.categories.forEach((categoryGroup) => {
                    md.push((0, elements_1.heading)(subHeadingLevel + 1, categoryGroup.title) + '\n');
                    md.push(getGroup(context, categoryGroup, inline) + '\n');
                });
            }
            else {
                const hasChildren = reflectionGroup.children.some((child) => Boolean(child.url));
                if (inline) {
                    md.push(`- [${context.groupTitle(reflectionGroup.title)}](${context.relativeURL((_a = context.page) === null || _a === void 0 ? void 0 : _a.url)}#${(0, utils_1.slugify)(context.groupTitle(reflectionGroup.title)).toLowerCase()})`);
                    if (hasChildren) {
                        md.push((0, elements_1.indentBlock)(getGroup(context, reflectionGroup, inline) + '\n'));
                    }
                }
                else {
                    md.push((0, elements_1.heading)(subHeadingLevel, context.groupTitle(reflectionGroup.title)) + '\n');
                    md.push(getGroup(context, reflectionGroup, inline) + '\n');
                }
            }
        });
    }
    return md.join('\n');
}
exports.reflectionIndex = reflectionIndex;
function getGroup(context, group, inline) {
    if (!inline && context.options.getValue('indexFormat') === 'table') {
        return getTable(context, group);
    }
    return getList(context, group);
}
function getTable(context, group) {
    const reflectionKind = group.children[0].kind;
    const headers = [
        typedoc_1.ReflectionKind.singularString(reflectionKind),
        context.getTextContent('label.description'),
    ];
    const rows = [];
    group.children.forEach((child) => {
        var _a;
        const row = [];
        row.push((0, elements_1.link)((0, utils_1.escapeChars)(child.name), context.relativeURL(child.url)));
        const comment = getComment(child);
        if ((_a = comment === null || comment === void 0 ? void 0 : comment.summary) === null || _a === void 0 ? void 0 : _a.length) {
            row.push((0, utils_1.formatTableDescriptionCol)(context.commentParts(comment.summary)).split('\n')[0]);
        }
        else {
            row.push('-');
        }
        rows.push(row);
    });
    return (0, elements_1.table)(headers, rows);
}
function getList(context, group) {
    const children = group.children
        .filter((child) => Boolean(child.url))
        .map((child) => {
        const name = child.name === 'constructor'
            ? `${child.signatures
                ? child.signatures[0].name
                : context.getTextContent('kind.constructor.singular')}`
            : child.name;
        return `- [${(0, utils_1.escapeChars)(name)}](${context.relativeURL(child.url)})`;
    });
    return children.join('\n');
}
function getComment(declaration) {
    var _a, _b, _c, _d, _e;
    if ((_a = declaration.signatures) === null || _a === void 0 ? void 0 : _a.length) {
        return declaration.signatures[0].comment;
    }
    if ((_d = (_c = (_b = declaration.type) === null || _b === void 0 ? void 0 : _b.declaration) === null || _c === void 0 ? void 0 : _c.signatures) === null || _d === void 0 ? void 0 : _d.length) {
        return (_e = declaration.type) === null || _e === void 0 ? void 0 : _e.declaration.signatures[0].comment;
    }
    return declaration.comment;
}
