"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parametersList = void 0;
const typedoc_1 = require("typedoc");
const elements_1 = require("../../../support/elements");
const utils_1 = require("../../../support/utils");
/**
 * @category Partials
 */
function parametersList(context, parameters) {
    const parseParams = (current, acc) => {
        var _a, _b, _c, _d;
        const shouldFlatten = ((_b = (_a = current.type) === null || _a === void 0 ? void 0 : _a.declaration) === null || _b === void 0 ? void 0 : _b.kind) === typedoc_1.ReflectionKind.TypeLiteral &&
            ((_d = (_c = current.type) === null || _c === void 0 ? void 0 : _c.declaration) === null || _d === void 0 ? void 0 : _d.children);
        return shouldFlatten
            ? [...acc, current, ...flattenParams(current)]
            : [...acc, current];
    };
    const flattenParams = (current) => {
        var _a, _b, _c;
        return (_c = (_b = (_a = current.type) === null || _a === void 0 ? void 0 : _a.declaration) === null || _b === void 0 ? void 0 : _b.children) === null || _c === void 0 ? void 0 : _c.reduce((acc, child) => {
            const childObj = {
                ...child,
                name: `${current.name}\\.${child.name}`,
            };
            return parseParams(childObj, acc);
        }, []);
    };
    const parsedParams = parameters.reduce((acc, current) => parseParams(current, acc), []);
    const firstOptionalParamIndex = parameters.findIndex((parameter) => parameter.flags.isOptional);
    const rows = [];
    parsedParams.forEach((parameter, i) => {
        const row = [];
        const isOptional = parameter.flags.isOptional ||
            (firstOptionalParamIndex !== -1 && i > firstOptionalParamIndex);
        const rest = parameter.flags.isRest ? '...' : '';
        const optional = isOptional ? '?' : '';
        const name = `${(0, utils_1.escapeChars)(parameter.name)}${optional}`;
        const identifier = [(0, elements_1.bold)(name)];
        if (parameter.type) {
            identifier.push(': ' + context.someType(parameter.type));
        }
        if (parameter.defaultValue) {
            identifier.push('= ' + getDefaultValue(parameter));
        }
        row.push(`• ${rest}${identifier.join('')}`);
        if (parameter.comment) {
            row.push(context.comment(parameter.comment));
        }
        rows.push(row.join('\n\n'));
    });
    return rows.join('\n\n');
}
exports.parametersList = parametersList;
function getDefaultValue(parameter) {
    return parameter.defaultValue && parameter.defaultValue !== '...'
        ? (0, elements_1.backTicks)(parameter.defaultValue)
        : (0, elements_1.backTicks)('undefined');
}
