"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.declarationMemberIdentifier = void 0;
const elements_1 = require("../../../support/elements");
const utils_1 = require("../../../support/utils");
const helpers_1 = require("../../helpers");
/**
 * @category Partials
 */
function declarationMemberIdentifier(context, reflection) {
    var _a;
    const md = [];
    const useCodeBlocks = context.options.getValue('useCodeBlocks');
    const declarationType = (0, helpers_1.getDeclarationType)(reflection);
    const prefix = [];
    const modifiers = reflection.flags.filter((flag) => flag !== 'Optional' && !reflection.flags.isRest);
    if (modifiers.length) {
        prefix.push(modifiers.map((flag) => (0, elements_1.bold)((0, elements_1.backTicks)(flag.toLowerCase()))).join(' '));
    }
    if (reflection.flags.isRest) {
        prefix.push('...');
    }
    if (useCodeBlocks &&
        (0, helpers_1.isGroupKind)(reflection) &&
        helpers_1.KEYWORD_MAP[reflection.kind]) {
        prefix.push(helpers_1.KEYWORD_MAP[reflection.kind]);
    }
    if (prefix.length) {
        md.push(prefix.join(' ') + ' ');
    }
    const name = [];
    if (reflection.getSignature) {
        name.push((0, elements_1.backTicks)('get') + ' ');
    }
    if (reflection.setSignature) {
        name.push((0, elements_1.backTicks)('set') + ' ');
    }
    name.push((0, elements_1.bold)((0, utils_1.escapeChars)(reflection.name)));
    if (reflection.typeParameters) {
        name.push(`\\<${(_a = reflection.typeParameters) === null || _a === void 0 ? void 0 : _a.map((typeParameter) => (0, elements_1.backTicks)(typeParameter.name)).join(', ')}\\>`);
    }
    if (reflection.flags.isOptional) {
        name.push('?');
    }
    if (declarationType) {
        name.push(': ');
    }
    md.push(name.join(''));
    if (declarationType) {
        md.push(context.someType(declarationType));
    }
    if (reflection.defaultValue && reflection.defaultValue !== '...') {
        md.push(` = \`${(0, utils_1.stripLineBreaks)((0, utils_1.stripComments)(reflection.defaultValue))}\``);
    }
    if (useCodeBlocks) {
        md.push(';');
    }
    const result = md.join('');
    return useCodeBlocks ? (0, elements_1.codeBlock)(result) : `> ${result}`;
}
exports.declarationMemberIdentifier = declarationMemberIdentifier;
