"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.declarationMember = void 0;
const typedoc_1 = require("typedoc");
const elements_1 = require("../../../support/elements");
/**
 * @category Partials
 */
function declarationMember(context, declaration, headingLevel, nested = false) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const md = [];
    md.push(context.declarationMemberIdentifier(declaration));
    const typeDeclaration = (_a = declaration.type) === null || _a === void 0 ? void 0 : _a.declaration;
    if (!((_b = typeDeclaration === null || typeDeclaration === void 0 ? void 0 : typeDeclaration.signatures) === null || _b === void 0 ? void 0 : _b.every((signature) => Boolean(signature.comment)))) {
        if (declaration.comment) {
            md.push(context.comment(declaration.comment, headingLevel));
        }
    }
    if (declaration.type instanceof typedoc_1.IntersectionType) {
        (_d = (_c = declaration.type) === null || _c === void 0 ? void 0 : _c.types) === null || _d === void 0 ? void 0 : _d.forEach((intersectionType) => {
            if (intersectionType instanceof typedoc_1.ReflectionType) {
                md.push((0, elements_1.heading)(headingLevel, 'Type declaration'));
                md.push(context.typeDeclarationMember(intersectionType.declaration, headingLevel));
            }
        });
    }
    if (declaration.type instanceof typedoc_1.ReferenceType &&
        ((_e = declaration.type.typeArguments) === null || _e === void 0 ? void 0 : _e.length)) {
        if (declaration.type.typeArguments[0] instanceof typedoc_1.ReflectionType) {
            md.push((0, elements_1.heading)(headingLevel, context.getTextContent('label.typeDeclaration')));
            md.push(context.typeDeclarationMember(declaration.type.typeArguments[0].declaration, headingLevel));
        }
    }
    if (declaration.typeParameters) {
        md.push((0, elements_1.heading)(headingLevel, context.getTextContent('kind.typeParameter.plural')));
        if (context.options.getValue('parametersFormat') === 'table') {
            md.push(context.typeParametersTable(declaration.typeParameters));
        }
        else {
            md.push(context.typeParametersList(declaration.typeParameters, headingLevel + 1));
        }
    }
    if (typeDeclaration) {
        if (typeDeclaration === null || typeDeclaration === void 0 ? void 0 : typeDeclaration.indexSignature) {
            md.push((0, elements_1.heading)(headingLevel, context.getTextContent('label.indexSignature')));
            md.push(context.indexSignatureTitle(typeDeclaration.indexSignature));
        }
        if ((_f = typeDeclaration === null || typeDeclaration === void 0 ? void 0 : typeDeclaration.signatures) === null || _f === void 0 ? void 0 : _f.length) {
            typeDeclaration.signatures.forEach((signature) => {
                md.push(context.signatureMember(signature, headingLevel, true));
            });
        }
        if ((_g = typeDeclaration === null || typeDeclaration === void 0 ? void 0 : typeDeclaration.children) === null || _g === void 0 ? void 0 : _g.length) {
            if (!nested && ((_h = typeDeclaration === null || typeDeclaration === void 0 ? void 0 : typeDeclaration.children) === null || _h === void 0 ? void 0 : _h.length)) {
                md.push((0, elements_1.heading)(headingLevel, context.getTextContent('label.typeDeclaration')));
                md.push(context.typeDeclarationMember(typeDeclaration, headingLevel));
            }
        }
    }
    md.push(context.inheritance(declaration, headingLevel));
    if (!nested &&
        declaration.sources &&
        !context.options.getValue('disableSources')) {
        md.push(context.sources(declaration, headingLevel));
    }
    return md.join('\n\n');
}
exports.declarationMember = declarationMember;
