"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.member = void 0;
const typedoc_1 = require("typedoc");
const elements_1 = require("../../../support/elements");
const utils_1 = require("../../../support/utils");
const helpers_1 = require("../../helpers");
/**
 * @category Partials
 */
function member(context, reflection, headingLevel, nested = false) {
    const md = [];
    if (context.options.getValue('namedAnchors')) {
        md.push(`<a id="${reflection.anchor}" name="${reflection.anchor}"></a>`);
    }
    if (!reflection.hasOwnDocument &&
        !(reflection.kind === typedoc_1.ReflectionKind.Constructor)) {
        const memberName = (0, helpers_1.getMemberTitle)(reflection);
        const memberHeading = context
            .getTextContent('title.member')
            .replace('{name}', memberName)
            .replace('{kind}', context.kindString(reflection.kind));
        md.push((0, elements_1.heading)(headingLevel, memberHeading));
    }
    const getMember = (reflection) => {
        var _a;
        if ([
            typedoc_1.ReflectionKind.Class,
            typedoc_1.ReflectionKind.Interface,
            typedoc_1.ReflectionKind.Enum,
        ].includes(reflection.kind)) {
            return context.reflectionMember(reflection, headingLevel + 1);
        }
        if (reflection.kind === typedoc_1.ReflectionKind.Constructor) {
            return context.constructorMember(reflection, headingLevel);
        }
        if (reflection.kind === typedoc_1.ReflectionKind.Accessor) {
            return context.accessorMember(reflection, headingLevel + 1);
        }
        if (reflection.signatures) {
            return (_a = reflection.signatures) === null || _a === void 0 ? void 0 : _a.map((signature) => {
                var _a, _b;
                const signatureMd = [];
                const multipleSignatures = reflection.signatures && ((_a = reflection.signatures) === null || _a === void 0 ? void 0 : _a.length) > 1;
                if (multipleSignatures) {
                    signatureMd.push((0, elements_1.heading)(headingLevel + 1, `${(0, utils_1.escapeChars)(signature.name)}(${(_b = signature.parameters) === null || _b === void 0 ? void 0 : _b.map((param) => param.name).join(', ')})`));
                }
                signatureMd.push(context.signatureMember(signature, multipleSignatures ? headingLevel + 2 : headingLevel + 1, nested));
                return signatureMd.join('\n\n');
            }).join('\n\n');
        }
        if (reflection instanceof typedoc_1.ReferenceReflection) {
            return context.referenceMember(reflection);
        }
        return context.declarationMember(reflection, headingLevel + 1, nested);
    };
    const member = getMember(reflection);
    if (member) {
        md.push(member);
    }
    return md.join('\n\n');
}
exports.member = member;
