"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.reflectionMember = void 0;
const elements_1 = require("../../../support/elements");
const helpers_1 = require("../../helpers");
/**
 * @category Partials
 */
function reflectionMember(context, reflection, headingLevel) {
    var _a;
    const md = [];
    if (reflection.comment) {
        md.push(context.comment(reflection.comment, headingLevel));
    }
    if ((_a = reflection.typeHierarchy) === null || _a === void 0 ? void 0 : _a.next) {
        md.push(context.memberHierarchy(reflection.typeHierarchy, headingLevel));
    }
    if (reflection.typeParameters) {
        md.push((0, elements_1.heading)(headingLevel, context.getTextContent('kind.typeParameter.plural')));
        if (context.options.getValue('parametersFormat') === 'table') {
            md.push(context.typeParametersTable(reflection.typeParameters));
        }
        else {
            md.push(context.typeParametersList(reflection.typeParameters, headingLevel + 1));
        }
    }
    if (reflection.implementedTypes) {
        md.push((0, elements_1.heading)(headingLevel, context.getTextContent('label.implements')));
        md.push((0, elements_1.unorderedList)(reflection.implementedTypes.map((implementedType) => context.someType(implementedType))));
    }
    if ('signatures' in reflection && reflection.signatures) {
        reflection.signatures.forEach((signature) => {
            md.push(context.signatureMember(signature, headingLevel));
        });
    }
    if ('indexSignature' in reflection && reflection.indexSignature) {
        md.push((0, elements_1.heading)(headingLevel, context.getTextContent('label.indexable')));
        md.push(context.indexSignatureTitle(reflection.indexSignature));
    }
    if ((0, helpers_1.hasIndex)(reflection)) {
        const isAbsolute = (0, helpers_1.isAbsoluteIndex)(reflection);
        if (isAbsolute) {
            md.push((0, elements_1.heading)(headingLevel, context.getTextContent('label.index')));
        }
        md.push(context.reflectionIndex(reflection, false, isAbsolute ? headingLevel + 1 : headingLevel));
    }
    md.push(context.members(reflection, headingLevel));
    return md.join('\n\n');
}
exports.reflectionMember = reflectionMember;
