"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.signatureMemberIdentifier = void 0;
const elements_1 = require("../../../support/elements");
const utils_1 = require("../../../support/utils");
const helpers_1 = require("../../helpers");
/**
 * @category Partials
 */
function signatureMemberIdentifier(context, signature, opts) {
    var _a;
    const md = [];
    const DEFAULT_OPTIONS = {
        accessor: null,
        useCodeBlocks: false,
    };
    const options = { ...DEFAULT_OPTIONS, ...opts };
    const useCodeBlocks = context.options.getValue('useCodeBlocks');
    if (useCodeBlocks &&
        (0, helpers_1.isGroupKind)(signature.parent) &&
        helpers_1.KEYWORD_MAP[signature.parent.kind]) {
        md.push(helpers_1.KEYWORD_MAP[signature.parent.kind] + ' ');
    }
    if (options === null || options === void 0 ? void 0 : options.accessor) {
        md.push((0, elements_1.bold)((0, elements_1.backTicks)(options === null || options === void 0 ? void 0 : options.accessor)) + ' ');
    }
    if (signature.parent && ((_a = signature.parent.flags) === null || _a === void 0 ? void 0 : _a.length) > 0) {
        md.push(signature.parent.flags
            .map((flag) => (0, elements_1.bold)((0, elements_1.backTicks)(flag.toLowerCase())))
            .join(' ') + ' ');
    }
    if (!['__call', '__type'].includes(signature.name)) {
        md.push((0, elements_1.bold)((0, utils_1.escapeChars)(signature.name)));
    }
    if (signature.typeParameters) {
        md.push(`\\<${signature.typeParameters
            .map((typeParameter) => (0, elements_1.backTicks)(typeParameter.name))
            .join(', ')}\\>`);
    }
    md.push((0, helpers_1.getSignatureParameters)(signature.parameters || [], useCodeBlocks));
    if (signature.type) {
        md.push(`: ${context.someType(signature.type)}`);
    }
    const result = md.join('');
    return useCodeBlocks ? (0, elements_1.codeBlock)(result) : `> ${result}`;
}
exports.signatureMemberIdentifier = signatureMemberIdentifier;
