"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.signatureMember = void 0;
const typedoc_1 = require("typedoc");
const elements_1 = require("../../../support/elements");
/**
 * @category Partials
 */
function signatureMember(context, signature, headingLevel, nested = false, accessor) {
    var _a, _b;
    const md = [];
    if (!nested) {
        md.push(context.signatureMemberIdentifier(signature, {
            accessor,
        }));
    }
    if (signature.comment) {
        md.push(context.comment(signature.comment, headingLevel, true, false));
    }
    if (((_a = signature.typeParameters) === null || _a === void 0 ? void 0 : _a.length) &&
        !signature.kindOf(typedoc_1.ReflectionKind.ConstructorSignature)) {
        md.push((0, elements_1.heading)(headingLevel, context.getTextContent('kind.typeParameter.plural')));
        if (context.options.getValue('parametersFormat') === 'table') {
            md.push(context.typeParametersTable(signature.typeParameters));
        }
        else {
            md.push(context.typeParametersList(signature.typeParameters, headingLevel + 1));
        }
    }
    if ((_b = signature.parameters) === null || _b === void 0 ? void 0 : _b.length) {
        md.push((0, elements_1.heading)(headingLevel, context.getTextContent('kind.parameter.plural')));
        if (context.options.getValue('parametersFormat') === 'table') {
            md.push(context.parametersTable(signature.parameters));
        }
        else {
            md.push(context.parametersList(signature.parameters));
        }
    }
    if (signature.type) {
        md.push(context.signatureMemberReturns(signature, headingLevel));
    }
    md.push(context.inheritance(signature, headingLevel));
    if (signature.comment) {
        md.push(context.comment(signature.comment, headingLevel, false, true));
    }
    if (!nested &&
        signature.sources &&
        !context.options.getValue('disableSources')) {
        md.push(context.sources(signature, headingLevel));
    }
    return md.join('\n\n');
}
exports.signatureMember = signatureMember;
