"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.propertiesTable = void 0;
const elements_1 = require("../../../support/elements");
const utils_1 = require("../../../support/utils");
const helpers_1 = require("../../helpers");
/**
 * @category Partials
 */
function propertiesTable(context, props, isEventProps = false) {
    const modifiers = props.map((param) => (0, helpers_1.getModifier)(param));
    const hasModifiers = modifiers.some((value) => Boolean(value));
    const hasOverrides = props.some((prop) => Boolean(prop.overwrites));
    const hasInheritance = props.some((prop) => Boolean(prop.inheritedFrom));
    const hasComments = props.some((prop) => { var _a, _b, _c, _d; return ((_b = (_a = prop.comment) === null || _a === void 0 ? void 0 : _a.blockTags) === null || _b === void 0 ? void 0 : _b.length) || ((_d = (_c = prop === null || prop === void 0 ? void 0 : prop.comment) === null || _c === void 0 ? void 0 : _c.summary) === null || _d === void 0 ? void 0 : _d.length); });
    const headers = [];
    if (hasModifiers) {
        headers.push(context.getTextContent('label.modifier'));
    }
    headers.push(isEventProps
        ? context.getTextContent('kind.event.singular')
        : context.getTextContent('kind.property.singular'));
    headers.push(context.getTextContent('label.type'));
    if (hasComments) {
        headers.push(context.getTextContent('label.description'));
    }
    if (hasOverrides) {
        headers.push(context.getTextContent('label.overrides'));
    }
    if (hasInheritance) {
        headers.push(context.getTextContent('label.inheritedFrom'));
    }
    const rows = [];
    const declarations = (0, helpers_1.flattenDeclarations)(props);
    declarations.forEach((property, index) => {
        var _a, _b, _c, _d;
        const propertyType = (0, helpers_1.getDeclarationType)(property);
        const row = [];
        if (hasModifiers) {
            row.push((0, elements_1.backTicks)(modifiers[index] || 'public'));
        }
        const nameColumn = [];
        if (context.options.getValue('namedAnchors') && property.anchor) {
            nameColumn.push(`<a id="${property.anchor}" name="${property.anchor}"></a>`);
        }
        const propertyName = `${property.name}${property.flags.isOptional ? '?' : ''}`;
        if (property.isDeprecated && property.isDeprecated()) {
            nameColumn.push((0, elements_1.strikeThrough)((0, elements_1.backTicks)(propertyName)));
        }
        else {
            nameColumn.push((0, elements_1.backTicks)(propertyName));
        }
        row.push(nameColumn.join(' '));
        if (propertyType) {
            row.push((0, utils_1.stripLineBreaks)(context.someType(propertyType), false));
        }
        if (hasComments) {
            const hasComment = ((_b = (_a = property.comment) === null || _a === void 0 ? void 0 : _a.blockTags) === null || _b === void 0 ? void 0 : _b.length) ||
                ((_d = (_c = property === null || property === void 0 ? void 0 : property.comment) === null || _c === void 0 ? void 0 : _c.summary) === null || _d === void 0 ? void 0 : _d.length);
            const comments = property === null || property === void 0 ? void 0 : property.comment;
            if (hasComment && comments) {
                row.push((0, utils_1.stripLineBreaks)((0, utils_1.formatTableDescriptionCol)(context.comment(comments))));
            }
            else {
                row.push('-');
            }
        }
        if (hasOverrides) {
            row.push(context.inheritance(property, -1) || '-');
        }
        if (hasInheritance) {
            row.push(context.inheritance(property, -1) || '-');
        }
        rows.push(row);
    });
    return (0, elements_1.table)(headers, rows);
}
exports.propertiesTable = propertiesTable;
