"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UrlsContext = void 0;
const path = __importStar(require("path"));
const typedoc_1 = require("typedoc");
const maps_1 = require("../options/maps");
const url_mapping_1 = require("../plugin/url-mapping");
const utils_1 = require("../support/utils");
const helpers_1 = require("./helpers");
class UrlsContext {
    constructor(theme, project, renderer, options) {
        this.theme = theme;
        this.project = project;
        this.renderer = renderer;
        this.options = options;
        this.urls = [];
        this.anchors = {};
    }
    /**
     * Map the models of the given project to the desired output files.
     * Based on TypeDoc DefaultTheme.getUrls()
     *
     * @param project  The project whose urls should be generated.
     */
    getUrls() {
        var _a, _b, _c, _d, _e;
        const preserveReadme = Boolean(this.project.readme) && !this.options.getValue('mergeReadme');
        const preserveModulesPage = (((_a = this.project) === null || _a === void 0 ? void 0 : _a.groups) &&
            Boolean((_c = (_b = this.project) === null || _b === void 0 ? void 0 : _b.groups[0]) === null || _c === void 0 ? void 0 : _c.children.find((child) => child.name === this.options.getValue('entryModule')))) ||
            false;
        const isPackages = this.options.getValue('entryPointStrategy') ===
            typedoc_1.EntryPointStrategy.Packages;
        const entryFileName = this.options.getValue('entryFileName');
        const indexFilename = (0, helpers_1.getIndexFileName)(this.project, isPackages);
        this.project.url = preserveReadme
            ? indexFilename
            : preserveModulesPage
                ? indexFilename
                : this.options.getValue('entryFileName');
        if (preserveReadme) {
            this.urls.push(new url_mapping_1.UrlMapping(preserveModulesPage ? 'readme_.md' : entryFileName, this.project, this.theme.readmeTemplate));
            this.urls.push(new url_mapping_1.UrlMapping(indexFilename, this.project, this.theme.projectTemplate));
        }
        else {
            this.urls.push(new url_mapping_1.UrlMapping(preserveModulesPage ? indexFilename : entryFileName, this.project, this.theme.projectTemplate));
        }
        if (isPackages) {
            if (((_d = Object.keys(this.renderer.packages)) === null || _d === void 0 ? void 0 : _d.length) === 1) {
                this.buildUrlsFromProject(this.project);
            }
            else {
                (_e = this.project.children) === null || _e === void 0 ? void 0 : _e.forEach((projectChild) => {
                    this.buildUrlsFromPackage(projectChild);
                });
            }
        }
        else {
            this.buildUrlsFromProject(this.project);
        }
        return this.urls;
    }
    buildUrlsFromPackage(projectChild) {
        const entryFileName = this.options.getValue('entryFileName');
        const preservePackageReadme = Boolean(projectChild.readme) && !this.options.getValue('mergeReadme');
        const packagesIndex = (0, helpers_1.getIndexFileName)(projectChild);
        const packageMeta = this.renderer.packages[projectChild.name];
        const outputFileStrategy = (packageMeta === null || packageMeta === void 0 ? void 0 : packageMeta.outputFileStrategy) ||
            this.options.getValue('outputFileStrategy');
        const url = `${projectChild.name}/${preservePackageReadme ? packagesIndex : entryFileName}`;
        if (preservePackageReadme) {
            this.urls.push(new url_mapping_1.UrlMapping(`${path.dirname(url)}/${entryFileName}`, projectChild, this.theme.readmeTemplate));
        }
        this.urls.push(new url_mapping_1.UrlMapping(url, projectChild, this.theme.projectTemplate));
        projectChild.url = url;
        this.buildUrlsFromProject(projectChild, url, outputFileStrategy);
    }
    /**
     *
     * @param project
     * @param isPackage
     */
    buildUrlsFromProject(project, parentUrl, outputFileStrategy) {
        var _a;
        (_a = project.groups) === null || _a === void 0 ? void 0 : _a.forEach((projectGroup) => {
            var _a;
            (_a = projectGroup.children) === null || _a === void 0 ? void 0 : _a.forEach((projectGroupChild) => {
                this.buildUrlsFromGroup(projectGroupChild, {
                    ...(parentUrl && { parentUrl }),
                    ...(outputFileStrategy && { outputFileStrategy }),
                });
            });
        });
    }
    buildUrlsFromGroup(reflection, options) {
        var _a;
        const mapping = this.theme.getTemplateMapping(reflection.kind, options.outputFileStrategy);
        if (mapping) {
            const directory = options.directory || mapping.directory;
            const urlPath = this.getUrlPath(reflection, {
                ...options,
                directory,
            });
            const url = this.getUrl(reflection, urlPath);
            this.urls.push(new url_mapping_1.UrlMapping(url, reflection, mapping.template));
            reflection.url = url;
            reflection.hasOwnDocument = true;
            (_a = reflection.groups) === null || _a === void 0 ? void 0 : _a.forEach((group) => {
                group.children.forEach((groupChild) => {
                    const mapping = this.theme.getTemplateMapping(groupChild.kind, options.outputFileStrategy);
                    this.buildUrlsFromGroup(groupChild, {
                        parentUrl: urlPath,
                        directory: (mapping === null || mapping === void 0 ? void 0 : mapping.directory) || null,
                        outputFileStrategy: options.outputFileStrategy,
                    });
                });
            });
        }
        else if (reflection.parent) {
            this.applyAnchorUrl(reflection, reflection.parent);
        }
    }
    getUrl(reflection, urlPath) {
        if (reflection.name === this.options.getValue('entryModule')) {
            return this.options.getValue('entryFileName');
        }
        if (this.options.getValue('outputFileStrategy') ===
            maps_1.OutputFileStrategy.Modules &&
            reflection.name === 'index') {
            return urlPath.replace('index.md', 'module.index.md');
        }
        return urlPath;
    }
    getUrlPath(reflection, options) {
        const alias = reflection.name
            .replace(/^_+/, '')
            .replace(/</, '-')
            .replace(/>/, '-');
        const parentDir = options.parentUrl
            ? path.dirname(options.parentUrl)
            : null;
        const dir = () => {
            if (reflection.kind === typedoc_1.ReflectionKind.Namespace) {
                return `${options.directory}/${alias}`;
            }
            if (reflection.kind === typedoc_1.ReflectionKind.Module) {
                return alias;
            }
            return options.directory
                ? options.directory
                : `${(0, utils_1.slugify)(typedoc_1.ReflectionKind.singularString(reflection.kind))}.${alias}`;
        };
        const filename = () => {
            if ([typedoc_1.ReflectionKind.Module, typedoc_1.ReflectionKind.Namespace].includes(reflection.kind) &&
                this.options.getValue('outputFileStrategy') ===
                    maps_1.OutputFileStrategy.Modules &&
                !this.childrenIncludeNamespaces(reflection)) {
                return null;
            }
            if ([typedoc_1.ReflectionKind.Module, typedoc_1.ReflectionKind.Namespace].includes(reflection.kind)) {
                return path.parse(this.options.getValue('entryFileName')).name;
            }
            return alias;
        };
        return ([parentDir, dir(), filename()].filter((part) => Boolean(part)).join('/') +
            '.md');
    }
    applyAnchorUrl(reflection, container) {
        var _a, _b;
        if (container.url) {
            if (!reflection.kindOf(typedoc_1.ReflectionKind.TypeLiteral)) {
                const anchorPrefix = this.options.getValue('anchorPrefix');
                const anchorId = this.getAnchorId(reflection);
                if (anchorId) {
                    if (!this.anchors[container.url]) {
                        this.anchors[container.url] = [];
                    }
                    this.anchors[container.url].push(anchorId);
                    const count = (_b = (_a = this.anchors[container.url]) === null || _a === void 0 ? void 0 : _a.filter((id) => id === anchorId)) === null || _b === void 0 ? void 0 : _b.length;
                    const anchorParts = [anchorId];
                    if (count > 1) {
                        anchorParts.push(`-${count - 1}`);
                    }
                    if (anchorPrefix) {
                        anchorParts.unshift(`${anchorPrefix}`);
                    }
                    reflection.url = container.url + '#' + anchorParts.join('');
                    reflection.anchor = anchorParts.join('');
                }
            }
            reflection.hasOwnDocument = false;
        }
        if (reflection.parent) {
            reflection.traverse((child) => {
                if (child instanceof typedoc_1.DeclarationReflection) {
                    this.applyAnchorUrl(child, container);
                }
            });
        }
    }
    getAnchorId(reflection) {
        const preserveAnchorCasing = this.options.getValue('preserveAnchorCasing');
        const anchorName = this.getAnchorName(reflection);
        if (anchorName) {
            return preserveAnchorCasing ? anchorName : anchorName.toLowerCase();
        }
        return null;
    }
    getAnchorName(reflection) {
        const htmlTableAnchors = this.options.getValue('namedAnchors');
        if (!htmlTableAnchors) {
            if ((reflection.kindOf(typedoc_1.ReflectionKind.Property) &&
                this.options.getValue('propertiesFormat') === 'table') ||
                (reflection.kindOf(typedoc_1.ReflectionKind.EnumMember) &&
                    this.options.getValue('enumMembersFormat') === 'table')) {
                return null;
            }
        }
        if (reflection.kindOf(typedoc_1.ReflectionKind.Constructor)) {
            return 'Constructors';
        }
        return reflection.name;
    }
    childrenIncludeNamespaces(reflection) {
        var _a;
        return (_a = reflection.children) === null || _a === void 0 ? void 0 : _a.some((child) => child.kind === typedoc_1.ReflectionKind.Namespace);
    }
}
exports.UrlsContext = UrlsContext;
